package org.cakelab.eval.mlatency;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.Vector;

import org.cakelab.eval.model.Measure;
import org.cakelab.eval.model.Plot;
import org.cakelab.eval.run.ResultsLogFile;
import org.cakelab.utils.FileSystem;

public class GNUPlot implements Plot {

	protected class MeasurePlot {
		protected Measure measure;
		protected int xcolumn;
		protected int ycolumn;

		MeasurePlot(Measure measure, int xcolumn, int ycolumn) {
			this.measure = measure;
			this.xcolumn = xcolumn;
			this.ycolumn = ycolumn;
		}
	}
	
	protected Vector<MeasurePlot> plots = new Vector<MeasurePlot>(1);
	protected String targetDir;
	protected String targetFilename;
	protected String xtitle;
	protected String ytitle;
	protected String units;
	protected String basedir;
	private boolean eps;
	private int xmin;
	private int xmax;
	private boolean setXRange = false;
	
	public GNUPlot(String baseDir, String targetFilename, String xtitle, String ytitle, String units) {
		this.basedir = baseDir;
		this.targetDir = FileSystem.createPlotsDirectory(baseDir);
		this.targetFilename = FileSystem.createFilename(targetFilename);
		this.xtitle = xtitle;
		this.ytitle = ytitle;
		this.units = units;
		FileSystem.createDirectory(baseDir);
	}

	
	public void plot() throws IOException {
		if (plots.size() == 0) return;

		final boolean APPEND = true;
		String script = targetDir + File.separator + targetFilename + ".dem";
		FileOutputStream fout = new FileOutputStream(script, !APPEND);
		PrintStream out = new PrintStream(fout);

		plotIntro(out);
		
		int i = 0;
		
		appendPlotCmd(out, plots.get(i));
		
		for (++i; i < plots.size(); i++) {
			out.print(", ");
			appendPlotCmd(out, plots.get(i));
		}
		plotOutro(out);

		out.close();
	}
	
	protected void plotIntro(PrintStream out) {
		out.println();
		if (eps) {
	        out.println("set terminal postscript eps colour");
	
	        out.println("set output \"" + targetFilename + ".eps\"");
		}
		if (setXRange) {
			out.println("set xrange [" + xmin + ":" + xmax + "]");
		}
		out.println("set xlabel \"" + xtitle + "\"");
		out.println("set ylabel \"" + ytitle + "[" + units + "]\"\n");
		out.print("plot ");
	}
	
	protected void appendPlotCmd(PrintStream out, MeasurePlot mp) {
		ResultsLogFile log = (ResultsLogFile) mp.measure.getResultLog();
		String logfile = log.getFileName(mp.measure);
		logfile = logfile.replace(basedir, "..");

		out.print("'" + logfile + "' using " + mp.xcolumn + ":" + mp.ycolumn + " title \"" + mp.measure.getName() + "\" with linespoints");
	}

	protected void plotOutro(PrintStream out) {
		out.println();
		if (!eps) out.println("pause -1 \"press enter to leave\"");
	}


	public void addMeasure(Measure measure, int xcolumn, int ycolumn) {
		plots.add(new MeasurePlot(measure, xcolumn, ycolumn));
	}


	@Override
	public void addMeasureErrorBars(Measure measure, int xcolumn, int ycolumn,
			int ylowcol, int yhighcol) {
		throw new Error("NOT IMPLEMENTED HERE use GNUPlotErrorBars");
	}


	public void setEPS(boolean eps) {
		this.eps = eps;
	}


	public void setXRange(int xmin, int xmax) {
		this.xmin = xmin;
		this.xmax = xmax;
		this.setXRange  = true;
	}
	
	
}
