package org.cakelab.eval.mlatency;


import org.cakelab.json.JSONCodec;
import org.cakelab.json.UnsafeAllocator;
import org.cakelab.utils.FileSystem;

public class MLatencyConfig {
	private int[] numThreads = {};
	private int minRuns = 3;
	private double accuracy = 1.0;
	private String executable = System.getProperty("user.home") + "/mlatency/mlatency";
	private String outdir = System.getProperty("user.home") + "/evaluate-latency-ef";
	private boolean epsOutput;
	

	/** Default constructor to set defaults. Defaults will be overridden by 
	 * anything defined in the config file.
	 */
	public MLatencyConfig() {
		numThreads = new int[]{2,4,8,16,32,60,64};
	}
	

	public int[] getNumThreads() {
		return numThreads;
	}


	public void setNumThreads(int[] numThreads) {
		this.numThreads = numThreads;
	}


	public int getMinRuns() {
		return minRuns;
	}


	public void setMinRuns(int minRuns) {
		this.minRuns = minRuns;
	}


	public double getAccuracy() {
		return accuracy;
	}


	public void setAccuracy(double accuracy) {
		this.accuracy = accuracy;
	}


	public String getExecutable() {
		return executable;
	}


	public void setExecutable(String executable) {
		this.executable = executable;
	}


	public String getOutdir() {
		return outdir;
	}


	public void setOutdir(String outdir) {
		this.outdir = outdir;
	}

	public boolean getEpsOutput() {
		return epsOutput;
	}


	public void setEpsOutput(boolean epsOutput) {
		this.epsOutput = epsOutput;
	}


	/* ********************************************************************
	 *  Read config from file
	 * ********************************************************************/
	
	
	/**
	 * Read config from file only. Any values not defined in the config file will be undefined!
	 * 
	 * @warning This method does not execute the default constructor of the 
	 * class. Thus, any values not defined in the config file will be undefined!
	 * @param cfgfile
	 * @return
	 * @throws Exception
	 */
	static MLatencyConfig read(String cfgfile) throws Exception {
		MLatencyConfig config = UnsafeAllocator.create().newInstance(MLatencyConfig.class);
		return read(config, cfgfile);
	}

	/**
	 * Read config from file into the given config instance. Any values defined in the config 
	 * file will override the default values of the given config instance!
	 * 
	 * @param config The config instance to receive the values of the config file.
	 * @param cfgfile The path of the config file to be read.
	 * @return The reference given with parameter config.
	 * @throws Exception
	 */
	public static MLatencyConfig read(MLatencyConfig config, String cfgfile) throws Exception {
		JSONCodec codec = new JSONCodec();
		codec.decodeObject(FileSystem.readText(cfgfile), config);
		return config;
	}
	
	
	


}
