package org.cakelab.eval.mlatency;


import org.cakelab.eval.model.LaunchConfig;
import org.cakelab.eval.model.LaunchConfigIterator;

public abstract class MLatencyLaunchConfigIterator implements LaunchConfigIterator {
	protected MLatencyLaunchConfig launch;
	protected int[] values;
	protected int current;
	protected String name;

	MLatencyLaunchConfigIterator(MLatencyLaunchConfig launch, int[] values, String name) {
		this.launch = launch;
		this.values = values;
		this.name = name;
		this.current = 0;
	}

	@Override
	public void remove() {
		throw new Error("UNSUPPORTED!");
	}

	public static LaunchConfigIterator createRW(MLatencyLaunchConfig launch,
			int[] numThreads) {
		LaunchConfigIterator rwIterator =  new MLatencyLaunchConfigIterator(launch, numThreads, "threads") {

			@Override
			public boolean hasNext() {
				return current < values.length;
			}

			@Override
			public LaunchConfig next() {
				int t = values[current++];
				int reader = t/2;
				int writer = t - reader;
				
				launch.setArgument("-r", reader);
				launch.setArgument("-w", writer);
				return launch;
			}

			@Override
			public String getName() {
				return name;
			}

		};
		return rwIterator;
	}
	
	public static LaunchConfigIterator createRO(MLatencyLaunchConfig launch, int[] numThreads) {
		LaunchConfigIterator rwIterator =  new MLatencyLaunchConfigIterator(launch, numThreads, "threads") {

			@Override
			public boolean hasNext() {
				return current < values.length;
			}

			@Override
			public LaunchConfig next() {
				int t = values[current++];
				
				launch.setArgument("-r", t);
				launch.setArgument("-w", 0);
				return launch;
			}

			@Override
			public String getName() {
				return name;
			}

		};
		return rwIterator;
	}
	
	public static LaunchConfigIterator createWO(MLatencyLaunchConfig launch, int[] numThreads) {
		LaunchConfigIterator rwIterator =  new MLatencyLaunchConfigIterator(launch, numThreads, "threads") {

			@Override
			public boolean hasNext() {
				return current < values.length;
			}

			@Override
			public LaunchConfig next() {
				int t = values[current++];
				
				launch.setArgument("-r", 0);
				launch.setArgument("-w", t);
				return launch;
			}

			@Override
			public String getName() {
				return name;
			}

		};
		return rwIterator;
	}

	public static LaunchConfigIterator createSR(MLatencyLaunchConfig launch,
			int[] numThreads) {
		LaunchConfigIterator iterator =  new MLatencyLaunchConfigIterator(launch, numThreads, "threads") {

			@Override
			public boolean hasNext() {
				return current < values.length;
			}

			@Override
			public LaunchConfig next() {
				int t = values[current++];
				
				launch.setArgument("-r", 1);
				launch.setArgument("-w", t - 1);
				return launch;
			}

			@Override
			public String getName() {
				return name;
			}

		};
		return iterator;
	}
	
	public static LaunchConfigIterator createSW(MLatencyLaunchConfig launch,
			int[] numThreads) {
		LaunchConfigIterator iterator =  new MLatencyLaunchConfigIterator(launch, numThreads, "threads") {

			@Override
			public boolean hasNext() {
				return current < values.length;
			}

			@Override
			public LaunchConfig next() {
				int t = values[current++];
				
				launch.setArgument("-r", t - 1);
				launch.setArgument("-w", 1);
				return launch;
			}

			@Override
			public String getName() {
				return name;
			}

		};
		return iterator;
	}

	public static LaunchConfigIterator createM(MLatencyLaunchConfig launch,
			int[] memsizes) {
		LaunchConfigIterator iterator =  new MLatencyLaunchConfigIterator(launch, memsizes, "memory size") {

			@Override
			public boolean hasNext() {
				return current < values.length;
			}

			@Override
			public LaunchConfig next() {
				int m = values[current++];
				int X = 10 * values[values.length-1] / m;
				X = X > 100 ? X : 100;
				launch.setArgument("-m", m);
				launch.setArgument("-X", X);
				return launch;
			}

			@Override
			public String getName() {
				return name;
			}

		};
		return iterator;
	}
	
	

}
