package org.cakelab.eval.mlatency;

import java.io.IOException;
import java.util.HashMap;
import java.util.Vector;

import org.cakelab.eval.model.ConfidenceEstimator;
import org.cakelab.eval.model.LaunchConfig;
import org.cakelab.eval.model.LaunchConfigIterator;
import org.cakelab.eval.model.Measure;
import org.cakelab.eval.model.DoubleValueExtractor;
import org.cakelab.eval.model.Plot;
import org.cakelab.eval.run.AbsoluteDeviation;
import org.cakelab.eval.run.ResultsLogFile;
import org.cakelab.eval.run.MeasureMeanDouble;
import org.cakelab.eval.run.RelativeAccuracy;
import org.cakelab.eval.run.AbsoluteAccuracy;
import org.cakelab.eval.run.ExecutionControl;
import org.cakelab.eval.run.RelativeConfidence;
import org.cakelab.eval.run.Report;
import org.cakelab.utils.Arguments;
import org.cakelab.utils.FileSystem;

public class Main {
	
	static MLatencyConfig config;
	static HashMap<String, Measure> allMeasures = new HashMap<String, Measure>(16);

	public static void main(String[] args) throws IOException {
		
		config = readConfig();
		
		
		String[] arguments = new String[]{
				"-X", "1000000", 
				"-m", "64",
				"-r", "0",
				"-w", "0"
		};
		
		int[] numThreads = config.getNumThreads();

		
		evaluateWO("WaR", numThreads, Arguments.merge(arguments, new String[]{"-d"}));
		evaluateWO("WaRa", numThreads, Arguments.merge(arguments, new String[]{"-a"}));
		evaluateWO("WaRe", numThreads, Arguments.merge(arguments, new String[]{"-e"}));
		evaluateWO("WaRf", numThreads, Arguments.merge(arguments, new String[]{"-f"}));
		
		multiplotThreads("WaRaef", "WaRa/avg", "WaRe/avg", "WaRf/avg");

		evaluateSR("SRa", numThreads, Arguments.merge(arguments, new String[]{"-a"}));
		evaluateSW("SWa", numThreads, Arguments.merge(arguments, new String[]{"-a"}));

		
		evaluateRO("RO", numThreads, Arguments.merge(arguments, new String[]{}));
		evaluateWO("WO", numThreads, Arguments.merge(arguments, new String[]{}));
		evaluateRW("RW", numThreads, Arguments.merge(arguments, new String[]{}));

		multiplotThreads("RO-WO-RW", "RO/avg", "WO/avg", "RW/avg", "RW/reader", "RW/writer");

		multiplotThreads("WaR", "RO/avg", "WO/avg", "WaR/writer");
		multiplotThreads("WaR", "RO/avg", "WO/avg", "WaR/writer", "WaRa/writer");

		evaluateSR("SR", numThreads, Arguments.merge(arguments, new String[]{}));
		evaluateSW("SW", numThreads, Arguments.merge(arguments, new String[]{}));

		multiplotThreads("SR", "SR/reader", "SR/writer");
		multiplotThreads("SW", "SW/reader", "SW/writer");
		

		evaluateRW("RWp", numThreads, Arguments.merge(arguments, new String[]{"-p"}));
		evaluateRW("RWjp", numThreads, Arguments.merge(arguments, new String[]{"-j", "-p"}));

		multiplotThreads("RW-paired", "RW/avg", "RW/reader", "RW/writer", "RWp/avg", "RWp/reader", "RWp/writer");
		multiplotThreads("RW-paired-disjunct", "RWp/avg", "RWp/reader", "RWp/writer", "RWjp/avg", "RWjp/reader", "RWjp/writer");

		
		evaluateRW("RWj", numThreads, Arguments.merge(arguments, new String[]{"-j"}));

		evaluateSR("WaRaSR", numThreads, Arguments.merge(arguments, new String[]{"-a"}));
		evaluateSR("WaRaSW", numThreads, Arguments.merge(arguments, new String[]{"-a", "-R", "writer"}));
		
		multiplotThreads("WaRaSR", "WaRaSR/reader", "WaRaSR/writer", "SR/reader");
		multiplotThreads("WaRaSW", "WaRaSW/reader", "WaRaSW/writer", "WaRaSR/reader", "WaRaSR/writer");
		
		arguments = new String[]{
				"-X", "10000", 
				"-m", "128000",
				"-r", "0",
				"-w", "0"
		};
		evaluateRW("RWjp128k", numThreads, Arguments.merge(arguments, new String[]{"-j", "-p"}));
		multiplotThreads("RWjp128k", "RWjp128k/avg", "RWjp128k/reader", "RWjp128k/writer");
		evaluateRW("RWj128k", numThreads, Arguments.merge(arguments, new String[]{"-j"}));
		multiplotThreads("RWj128k", "RWj128k/avg", "RWj128k/reader", "RWj128k/writer");

		arguments = new String[]{
				"-X", "1000", 
				"-m", "1048576",
				"-r", "0",
				"-w", "0"
		};
		evaluateRW("RWjp1MB", numThreads, Arguments.merge(arguments, new String[]{"-j", "-p"}));
		multiplotThreads("RWjp1MB", "RWjp1MB/avg", "RWjp1MB/reader", "RWjp1MB/writer");
		evaluateRW("RWj1MB", numThreads, Arguments.merge(arguments, new String[]{"-j"}));
		multiplotThreads("RWj1MB", "RWj1MB/avg", "RWj1MB/reader", "RWj1MB/writer");

		
		arguments = new String[]{
				"-X", "1000", 
				"-m", "1048576",
				"-r", "0",
				"-w", "0"
		};
		evaluateRW("RWjpx1MB", numThreads, Arguments.merge(arguments, new String[]{"-j", "-p", "-x"}));
		multiplotThreads("RWjpx1MB", "RWjpx1MB/avg", "RWjpx1MB/reader", "RWjpx1MB/writer");
		evaluateRW("RWjx1MB", numThreads, Arguments.merge(arguments, new String[]{"-j", "-x"}));
		multiplotThreads("RWjx1MB", "RWjx1MB/avg", "RWjx1MB/reader", "RWjx1MB/writer");


		arguments = new String[]{
				"-X", "1000", 
				"-m", "2097152",
				"-r", "0",
				"-w", "0"
		};
		evaluateRW("RWjp2MB", numThreads, Arguments.merge(arguments, new String[]{"-j", "-p"}));
		multiplotThreads("RWjp2MB", "RWjp2MB/avg", "RWjp2MB/reader", "RWjp2MB/writer");
		evaluateRW("RWj2MB", numThreads, Arguments.merge(arguments, new String[]{"-j"}));
		multiplotThreads("RWj2MB", "RWj2MB/avg", "RWj2MB/reader", "RWj2MB/writer");
		
		arguments = new String[]{
				"-X", "1000", 
				"-m", "2097152",
				"-r", "0",
				"-w", "0"
		};
		evaluateRW("RWjpx2MB", numThreads, Arguments.merge(arguments, new String[]{"-j", "-p", "-x"}));
		multiplotThreads("RWjpx2MB", "RWjpx2MB/avg", "RWjpx2MB/reader", "RWjpx2MB/writer");
		evaluateRW("RWjx2MB", numThreads, Arguments.merge(arguments, new String[]{"-j", "-x"}));
		multiplotThreads("RWjx2MB", "RWjx2MB/avg", "RWjx2MB/reader", "RWjx2MB/writer");

		arguments = new String[]{
				"-X", "1000", 
				"-m", "0",
				"-r", "1",
				"-w", "1"
		};
		evaluate_M_iter("M", new int[]{64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, 65536, 131072, 262144, 524288, 1048576, 2097152, 4194304}, Arguments.merge(arguments, new String[]{"-W", "writer_global_data"}));
	
		evaluate_M_iter("Mx", new int[]{64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, 65536, 131072, 262144, 524288, 1048576, 2097152, 4194304}, Arguments.merge(arguments, new String[]{"-W", "writer_global_data", "-x"}));
		
		multiplotMemory("Mboth", "M/reader", "M/writer", "Mx/reader", "Mx/writer");

		
		arguments = new String[]{
				"-X", "1000", 
				"-m", "0",
				"-r", "2",
				"-w", "0"
		};
		evaluate_M_iter("Mj", new int[]{64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, 65536, 131072, 262144, 524288, 1048576, 2097152, 4194304}, Arguments.merge(arguments, new String[]{"-j"}));
		
		evaluate_M_iter("Mjx", new int[]{64, 128, 512, 1024, 2048, 4096, 8192, 16384, 32768, 65536, 131072, 262144, 524288, 1048576, 2097152, 4194304}, Arguments.merge(arguments, new String[]{"-j", "-x"}));
		
		multiplotMemory("Mjboth", "Mj/reader", "Mjx/reader");
		
		multiplotMemoryXRange(0, 65536, "Mjboth-lower", "M/reader", "Mx/reader", "Mj/reader", "Mjx/reader");

	}



	private static void multiplotThreads(String name, String... types) throws IOException {
		GNUPlot plot = new GNUPlot(config.getOutdir(), name, "threads", "latency", "ns");
		plot.setEPS(config.getEpsOutput());
		for (String t : types) {
			plot.addMeasure(allMeasures.get(t), 1, 3);
		}
		
		plot.plot();
		
	}

	private static void multiplotMemory(String name, String... types) throws IOException {
		GNUPlot plot = new GNUPlot(config.getOutdir(), name, "memory size [b]", "latency", "ns");
		plot.setEPS(config.getEpsOutput());
		for (String t : types) {
			plot.addMeasure(allMeasures.get(t), 1, 3);
		}
		
		plot.plot();
		
	}

	private static void multiplotMemoryXRange(int xmin, int xmax, String name, String... types) throws IOException {
		GNUPlot plot = new GNUPlot(config.getOutdir(), name, "memory size [b]", "latency", "ns");
		plot.setEPS(config.getEpsOutput());
		plot.setXRange(xmin, xmax);
		for (String t : types) {
			plot.addMeasure(allMeasures.get(t), 1, 3);
		}
		
		plot.plot();
		
	}

	private static void multiplotThreads_errorbars(String name, String... types) throws IOException {
		GNUPlotErrorBar plot = new GNUPlotErrorBar(config.getOutdir(), name, "threads", "latency", "ns");
		plot.setEPS(config.getEpsOutput());
		for (String t : types) {
			plot.addMeasureErrorBars(allMeasures.get(t), 1, 3, 10, 11);
		}
		
		plot.plot();
		
	}


	public static void evaluateT(String[] arguments, String evalType, LaunchConfigIterator iterator) {
		Vector<Measure> measures = new Vector<Measure>(3);
		DoubleValueExtractor extractor = new MLatencyTotalMeanExtractor();
		DoubleValueExtractor readerExtractor = new MLatencyFilteredMeanExtractor("reader");
		DoubleValueExtractor writerExtractor = new MLatencyFilteredMeanExtractor("writer");
		// max 10% of stddev deviation in the estimated mean if the next value would be an extreme value (== current min/max)
//		ConfidenceEstimator confidence = new RelativeAccuracy(10, 0.10);
		
		// max 20% of stddev deviation in the estimated mean
		ConfidenceEstimator confidence = new RelativeConfidence(0.90, config.getMinRuns(), config.getAccuracy());
		
		// first 2 digits have to be stable
//		ConfidenceEstimator confidence = new AbsoluteAccuracy(10, 2);
		
		// possible next deviation from mean within less than 0.02 units 
//		ConfidenceEstimator confidence = new AbsoluteDeviation(10, 0.02);
		
		


		
		try {
			
			MLatencyResultsLogThreads outOverall = new MLatencyResultsLogThreads(config.getOutdir());
			measures.add(new MeasureMeanDouble(evalType + "/avg", "ns", extractor, confidence, outOverall));
			MLatencyResultsLogThreads outReader = new MLatencyResultsLogThreads(config.getOutdir());
			measures.add(new MeasureMeanDouble(evalType + "/reader", "ns", readerExtractor, confidence, outReader));
			MLatencyResultsLogThreads outWriter = new MLatencyResultsLogThreads(config.getOutdir());
			measures.add(new MeasureMeanDouble(evalType + "/writer", "ns", writerExtractor, confidence, outWriter));
			MLatencyDataLog dataFileLog = new MLatencyDataLog(config.getOutdir());


			while (iterator.hasNext()) {
				LaunchConfig launch = iterator.next();
				
				ExecutionControl control = new ExecutionControl(launch, measures, dataFileLog);
				control.run();
			}

			Plot plot = new GNUPlot(config.getOutdir(), evalType, iterator.getName(), "latency", "ns");
			for (Measure measure : measures) {
				allMeasures.put(measure.getName(), measure);
				plot.addMeasure(measure, 1, 3);
			}
			plot.plot();
			
			
		} catch (Exception e) {
			Report.error(e.getMessage());
		}
	}
	
	public static void evaluateM(String[] arguments, String evalType, LaunchConfigIterator iterator) {
		Vector<Measure> measures = new Vector<Measure>(3);
		DoubleValueExtractor extractor = new MLatencyTotalMeanExtractor();
		DoubleValueExtractor readerExtractor = new MLatencyFilteredMeanExtractor("reader");
		DoubleValueExtractor writerExtractor = new MLatencyFilteredMeanExtractor("writer");
		// max 10% of stddev deviation in the estimated mean if the next value would be an extreme value (== current min/max)
//		ConfidenceEstimator confidence = new RelativeAccuracy(10, 0.10);
		
		// max 20% of stddev deviation in the estimated mean
		ConfidenceEstimator confidence = new RelativeConfidence(0.90, config.getMinRuns(), config.getAccuracy());
		
		// first 2 digits have to be stable
//		ConfidenceEstimator confidence = new AbsoluteAccuracy(10, 2);
		
		// possible next deviation from mean within less than 0.02 units 
//		ConfidenceEstimator confidence = new AbsoluteDeviation(10, 0.02);
		
		


		
		try {
			
			MLatencyResultsLogMemory outOverall = new MLatencyResultsLogMemory(config.getOutdir());
			measures.add(new MeasureMeanDouble(evalType + "/avg", "ns", extractor, confidence, outOverall));
			MLatencyResultsLogMemory outReader = new MLatencyResultsLogMemory(config.getOutdir());
			measures.add(new MeasureMeanDouble(evalType + "/reader", "ns", readerExtractor, confidence, outReader));
			MLatencyResultsLogMemory outWriter = new MLatencyResultsLogMemory(config.getOutdir());
			measures.add(new MeasureMeanDouble(evalType + "/writer", "ns", writerExtractor, confidence, outWriter));
			MLatencyDataLog dataFileLog = new MLatencyDataLog(config.getOutdir());


			while (iterator.hasNext()) {
				LaunchConfig launch = iterator.next();
				
				ExecutionControl control = new ExecutionControl(launch, measures, dataFileLog);
				control.run();
			}

			Plot plot = new GNUPlot(config.getOutdir(), evalType, iterator.getName(), "latency", "ns");
			for (Measure measure : measures) {
				allMeasures.put(measure.getName(), measure);
				plot.addMeasure(measure, 1, 3);
			}
			plot.plot();
			
			
		} catch (Exception e) {
			Report.error(e.getMessage());
		}
	}

	private static MLatencyConfig readConfig() {
		config = new MLatencyConfig();
		String cfgdefault = "ef.cfg";
		String cfgfile = System.getProperty("org.cakelab.eval.config");
		try {
			if (cfgfile == null && FileSystem.exists(cfgdefault)) {
				cfgfile = cfgdefault;
			}
			
			if (cfgfile != null) {
				Report.info("reading config from '" + cfgfile + "'");
				MLatencyConfig.read(config, cfgfile);
			}
		} catch (Exception e) {
			Report.warn("config file : \"" + cfgfile + "\" ignored due to an exception stating: \"" + e.getMessage());
		}
		return config;
	}


	private static void evaluateRW(String name, int[] numThreads, String[] arguments) {
		MLatencyLaunchConfig launch = new MLatencyLaunchConfig(config.getExecutable(), arguments);
		LaunchConfigIterator rwIterator =  MLatencyLaunchConfigIterator.createRW(launch, numThreads);
		evaluateT(arguments, name, rwIterator);
	}

	private static void evaluateRO(String name, int[] numThreads, String[] arguments) {
		MLatencyLaunchConfig launch = new MLatencyLaunchConfig(config.getExecutable(), arguments);
		LaunchConfigIterator rwIterator =  MLatencyLaunchConfigIterator.createRO(launch, numThreads);
		evaluateT(arguments, name, rwIterator);
	}

	private static void evaluateWO(String name, int[] numThreads, String[] arguments) {
		MLatencyLaunchConfig launch = new MLatencyLaunchConfig(config.getExecutable(), arguments);
		LaunchConfigIterator rwIterator =  MLatencyLaunchConfigIterator.createWO(launch, numThreads);
		evaluateT(arguments, name, rwIterator);
	}

	private static void evaluateSR(String name, int[] numThreads, String[] arguments) {
		MLatencyLaunchConfig launch = new MLatencyLaunchConfig(config.getExecutable(), arguments);
		LaunchConfigIterator rwIterator =  MLatencyLaunchConfigIterator.createSR(launch, numThreads);
		evaluateT(arguments, name, rwIterator);
	}

	private static void evaluateSW(String name, int[] numThreads, String[] arguments) {
		MLatencyLaunchConfig launch = new MLatencyLaunchConfig(config.getExecutable(), arguments);
		LaunchConfigIterator rwIterator =  MLatencyLaunchConfigIterator.createSW(launch, numThreads);
		evaluateT(arguments, name, rwIterator);
	}

	private static void evaluate_M_iter(String name, int[] memsizes, String[] arguments) {
		MLatencyLaunchConfig launch = new MLatencyLaunchConfig(config.getExecutable(), arguments);
		LaunchConfigIterator rwIterator =  MLatencyLaunchConfigIterator.createM(launch, memsizes);
		evaluateM(arguments, name, rwIterator);
	}


}
