package org.cakelab.eval.model;

import java.io.IOException;
import java.io.InputStream;

import org.cakelab.utils.Text;
import org.cakelab.utils.FileSystem;

public abstract class LaunchConfig {
	protected String executable;
	protected String[] env;
	protected InputStream stdout;
	protected InputStream stderr;
	protected String[] arguments;
	
	
	public LaunchConfig(String executable, String[] arguments, String[] env) {
		this.executable = executable;
		this.arguments = arguments;
		this.env = env;
	}

	/** Executes the defined process and waits for its result. 
	 * @throws Error if the exit status of the executed process was not zero!
	 * */
	protected Process exec() throws IOException, InterruptedException {
		Process p = Runtime.getRuntime().exec(getCommandLine(), env);
		int status = p.waitFor();
		/* some java implementations seems to 'consume' the status in waitFor ... weird */
		if (p.exitValue() != 0 || status != 0) {
			FileSystem.dump2stream(p.getErrorStream(), System.err);
			throw new Error("execution failed. cmd: " + getCommandLine());
		}
		return p;
	}
	
	public abstract MeasurementRun execute() throws IOException, InterruptedException;

	public InputStream getStdout() {
		return stdout;
	}

	public InputStream getStderr() {
		return stderr;
	}

	public String getExecutable() {
		return executable;
	}

	public String getArgumentsString(String sep) {
		String args = "";
		for (int i = 0; i < arguments.length; i++) {
			args += arguments[i].trim();
			if (i < arguments.length-1) args += sep; 
		}
		return args;
	}

	public String getArgumentsString() {
		return getArgumentsString(Text.columnSeparator);
	}

	public String[] getArguments () {
		return arguments;
	}

	public void setArguments(String[] arguments) {
		this.arguments = arguments;
	}

	public String getCommandLine ()
	{
		return executable + Text.columnSeparator + getArgumentsString(Text.columnSeparator);
	}

	public Long getArgumentLong(String string, long defaultValue) {
		for (int i = 0; i < arguments.length -1; i++) {
			String arg = arguments[i];
			if (arg.equals(string)) {
				return Long.decode(arguments[i+1]);
			}
		}
		return defaultValue;
	}
}
