package org.cakelab.eval.model;

import java.io.IOException;

public abstract class Measure {
	private String name;
	private String units;
	private ResultsLog log;
	
	public Measure(String name, String units, ResultsLog log) throws IOException {
		super();
		this.name = name;
		this.units = units;
		this.log = log;
		
		log.clear(this);
	}
	
	/** Some name for the measure for display purposes */
	public String getName() {
		return name;
	}
	/** for displaying purposes only .. oh, and you might want to remember 
	 * yourself what unit this measure actually had ;) */
	public String getUnits() {
		return units;
	}
	
	/** Add a run to the set of measurement runs to be evaluated. */
	public abstract void add(MeasurementRun run) throws IOException;

	/** Called by the execution controller to determine whether the results 
	 * accuracy has been met and he can stop to do more measurement runs for 
	 * this measure.
	 * @return
	 */
	public abstract boolean isConfident();
	
	/** This returns the value calculated to determine the confidence. */
	public abstract double getError();
	
	/** this method discards any existing results and resets them 
	 * to initial value 
	 * @throws IOException */
	public void setup() throws IOException {
		log.open(this);
	}

	
	public void store(long timestamp, LaunchConfig launch) throws IOException {
		log.store(this, timestamp, launch);
	}

	public void tearDown() throws IOException {
		log.close(this);
	}
	
	public abstract Number[] getValues();

	public ResultsLog getResultLog() {
		return log;
	}

}
