package org.cakelab.eval.model;

import java.io.IOException;


/**
 * A results log contains the calculated results of a particular measure 
 * over a set of measurement runs of different launch configurations.
 * 
 * A result log is typically associated with one particular measure.
 * 
 * A log entry consists of one result of the measure for a set of runs of 
 * a particular launch configuration:
 * <ul>
 * <li>Start time: The time the set of runs of a particular launch configuration was started</li>
 * <li>Launch configuration: The data to identify the launch configuration used for this set of runs</li>
 * <li>Measure: The value(s) calculated for a particular measure</li>
 * </ul>
 * @author homac
 *
 */
public interface ResultsLog {

	/**
	 * @param measure Identifies the measure this log is associated with
	 * @throws IOException
	 */
	void open(Measure measure) throws IOException;
	
	/**
	 * Deletes all existing results for this measure stored in this log.
	 */
	public void clear(Measure measure) throws IOException;
	
	/**
	 * @param measure Identifies the measure this log is associated with
	 * @param timestamp
	 * @param launch
	 * @throws IOException
	 */
	void store(Measure measure, long timestamp, LaunchConfig launch) throws IOException;

	/**
	 * 
	 * @param measure Identifies the measure this log is associated with
	 * @throws IOException
	 */
	void close(Measure measure) throws IOException;

}
