package org.cakelab.eval.run;

import org.cakelab.eval.model.ConfidenceEstimator;
import org.cakelab.statistics.Statistics;

public abstract class EstimatedMeanError implements ConfidenceEstimator {
	protected int minSamples;
	
	EstimatedMeanError(int minSamples) {
		this.minSamples = minSamples > 1 ? minSamples : 1;
	}
	
	@Override
	public double getError(Statistics stats) {
		if (stats.getNum() < minSamples) return Double.NaN;
		
		/* We consider the next sample to be a new sample at the maximum or minimum. */
		double maxPosDev = stats.getMaxDouble() - stats.getMean();
		double maxNegDev = stats.getMinDouble() - stats.getMean();
		double extremeNextDeviation = (Math.abs(maxPosDev) > Math.abs(maxNegDev)) ? maxPosDev : maxNegDev;
		
		if (stats.getStddev() == 0) {
			return 0.0;
		}
		
		if (Double.isNaN(stats.getMean())) {
			return 0.0;
		}
		/* thus our next mean could be: */
		double possibleNextMean = (stats.getMean() * stats.getNum() + Math.abs(stats.getMean() + extremeNextDeviation)) / (stats.getNum() +1);
		
		double maxDeviation = Math.abs(possibleNextMean - stats.getMean());
		return maxDeviation;
	}

}
