package org.cakelab.eval.run;

import java.io.IOException;

import org.cakelab.eval.model.ConfidenceEstimator;
import org.cakelab.eval.model.DoubleValueExtractor;
import org.cakelab.eval.model.LaunchConfig;
import org.cakelab.eval.model.Measure;
import org.cakelab.eval.model.MeasurementRun;
import org.cakelab.eval.model.ResultsLog;
import org.cakelab.statistics.StatisticsDouble;
import org.cakelab.utils.DoubleArray;

public class MeasureMeanDouble extends Measure {
	private DoubleValueExtractor extractor;
	private DoubleArray values = new DoubleArray(128);
	private StatisticsDouble stats;
	private ConfidenceEstimator confidence; /* requested maximum size of the confidence interval at the given level */
	private ResultsLog log;

	/**
	 * 
	 * @param name
	 * @param units
	 * @param extractor
	 * @param level
	 * @param maxConfidenceIntervalRangeRatio Maximum size of the confidence interval given as ratio of the total value range (meaning: [min,max])
	 * @throws IOException 
	 */
	public MeasureMeanDouble (String name, String units, DoubleValueExtractor extractor, ConfidenceEstimator confidence, ResultsLog log) throws IOException {
		super(name, units, log);
		this.extractor = extractor;
		this.log = log;
		this.stats = new StatisticsDouble();
		this.confidence = confidence;
		
	}
	
		
	public void add(MeasurementRun run) throws IOException {
		values.add(extractor.getValue(run));
		stats.update(values);
	}
	
	public boolean isConfident() {
		return confidence.isConfident(stats);
	}


	@Override
	public void store(long timestamp, LaunchConfig launch) throws IOException {
		log.store(this, timestamp, launch);
	}


	@Override
	public void setup() throws IOException {
		stats.clear();
		values.clear();
		super.setup();
	}


	@Override
	public Number[] getValues() {
		return stats.getValues();
	}


	@Override
	public double getError() {
		return confidence.getError(stats);
	}


}
