package org.cakelab.eval.run;

import java.io.IOException;

import org.cakelab.eval.model.ConfidenceEstimator;
import org.cakelab.eval.model.LaunchConfig;
import org.cakelab.eval.model.LongValueExtractor;
import org.cakelab.eval.model.Measure;
import org.cakelab.eval.model.MeasurementRun;
import org.cakelab.eval.model.ResultsLog;
import org.cakelab.statistics.StatisticsLong;
import org.cakelab.utils.LongArray;


/**
 * This class serves the measurement of a mean based on long values with a 
 * accuracy evaluated by the given confidence estimator on the values provided by the given long value extractor.
 *
 */
public class MeasureMeanLong extends Measure {
	private LongValueExtractor extractor;
	private LongArray values = new LongArray(128);
	private StatisticsLong stats;
	private ConfidenceEstimator confidence; /* requested maximum size of the confidence interval at the given level */
	private ResultsLog log;
	/**
	 * 
	 * @param name
	 * @param units
	 * @param extractor
	 * @param level
	 * @param maxConfidenceIntervalRangeRatio Maximum size of the confidence interval given as ratio of the total value range (meaning: [min,max])
	 * @throws IOException 
	 */
	public MeasureMeanLong (String name, String units, LongValueExtractor extractor, ConfidenceEstimator confidence, ResultsLog log) throws IOException {
		super(name, units, log);
		this.extractor = extractor;
		this.log = log;
		this.stats = new StatisticsLong();
		this.confidence = confidence;
	}
	
		
	public void add(MeasurementRun run) throws IOException {
		values.add(extractor.getValue(run));
		stats.update(values);
		
		Report.info("stats: "  + stats.toString());
	}
	
	public boolean isConfident() {
		return confidence.isConfident(stats);
	}


	@Override
	public void store(long timestamp, LaunchConfig launch) throws IOException {
		log.store(this, timestamp, launch);
	}


	@Override
	public void setup() throws IOException {
		stats.clear();
		values.clear();
		super.setup();
	}


	@Override
	public Number[] getValues() {
		return stats.getValues();
	}


	@Override
	public double getError() {
		return confidence.getError(stats);
	}


	
}
