package org.cakelab.eval.run;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;

import org.cakelab.eval.model.LaunchConfig;
import org.cakelab.eval.model.Measure;
import org.cakelab.eval.model.ResultsLog;
import org.cakelab.utils.FileSystem;
import org.cakelab.utils.Text;

public class ResultsLogFile implements ResultsLog {
	private static final boolean APPEND = true;
	protected PrintStream out;
	private String basedir;
	private String resultsDir;
	
	
	public ResultsLogFile(String targetDir) throws FileNotFoundException {
		this.basedir = targetDir;
		this.resultsDir = FileSystem.createResultsLogDirectory(basedir);
	}

	@Override
	public void clear(Measure measure) {
		new File(getFileName(measure)).delete();
	}
	
	@Override
	public void open(Measure measure) throws IOException {
		String fn = getFileName(measure);
		String dir = FileSystem.getDirectory(fn);
		if (dir != null) FileSystem.createDirectory(dir);
		FileOutputStream fout = new FileOutputStream(getFileName(measure), APPEND);
		out = new PrintStream(fout);
	}

	@Override
	public void store(Measure measure, long timestamp, LaunchConfig launch) throws IOException {
		StringBuffer valuesStr = new StringBuffer();
		Number[] values = measure.getValues();
		for (Number val : values) {
			valuesStr.append(val);
			valuesStr.append(Text.columnSeparator);
		}
		out.println(timestamp + Text.tableSeparator + launch.getArgumentsString(Text.columnSeparator) + Text.tableSeparator + valuesStr.toString());
	}

	@Override
	public void close(Measure measure) throws IOException {
		out.close();
	}

	public String getFileName(Measure measure) {
		return resultsDir + File.separator + FileSystem.createFilename(measure.getName()) + ".result";
	}

}
