package org.cakelab.statistics;

import org.cakelab.utils.DoubleArray;

public class StatisticsDouble implements Statistics {


	long num; /* number of samples */
	double min; 
	double max;
	double mean;
	double median;
	double stddev;
	double variance;
	double avgabsdev;

	public void update(DoubleArray values){
		num = values.size();
		max = Double.MIN_VALUE;
		min = Double.MAX_VALUE;
		
		double sum = 0;
		
		
		//
		// Min, max and mean
		//
		
		for (int i = 0; i < num; i++) {
			double val = values.a[i];
			if (val < min) min = val;
			if (val > max) max = val;
			sum += val;
		}
		
		mean = sum/num;
		
		
		//
		// variance, standard deviation and average absolute deviation
		//
		double absdevsum = 0;
		double squaresum = 0;
		double dev;
		for (int i = 0; i < num; i++) {
			double val = values.a[i];
			dev = val-mean;
			absdevsum += Math.abs(dev);
			squaresum += dev*dev;
		}
		variance = squaresum/num;
		stddev = Math.sqrt(variance);
		avgabsdev = absdevsum/num;

		//
		// median
		//
		// TODO: instead of sorting each time, insert sorted on each new sample (Measure.add())
		values.sort();
		median = values.a[(int) (num/2)];
		
	}
	
	
	
	public static double mean(DoubleArray a) {
		double sum = 0;
		for (int i = 0; i < a.size(); i++) {
			sum += a.a[i];
		}
		return sum / a.size();
	}
	
	


	public long getNum() {
		return num;
	}



	public double getMinDouble() {
		return min;
	}

	public long getMinLong() {
		return (long)min;
	}



	public double getMaxDouble() {
		return max;
	}

	public long getMaxLong() {
		return (long)max;
	}



	public double getMean() {
		return mean;
	}



	public double getMedianDouble() {
		return median;
	}

	public long getMedianLong() {
		return (long)median;
	}



	public double getStddev() {
		return stddev;
	}



	public double getVariance() {
		return variance;
	}



	public double getAvgabsdev() {
		return avgabsdev;
	}
	
	public String toString() {
		StringBuffer result = new StringBuffer("num: " + num + ", ");
		result.append("mean: " + mean + ", ");
		result.append("median: " + median + ", ");
		result.append("variance: " + variance + ", ");
		result.append("stddev: " + stddev + ", ");
		result.append("avgabsdev: " + avgabsdev);
		result.append("min: " + min + ", ");
		result.append("max: " + max + ", ");
		return result.toString();
	}



	public void clear() {
		num = 0;
		max = Long.MIN_VALUE;
		min = Long.MAX_VALUE;
		mean = Double.NaN;
		median = Double.NaN;
		variance = Double.NaN;
		stddev = Double.NaN;
		avgabsdev = Double.NaN;
	}



	@Override
	public Number[] getValues() {
		Number[] values = new Number[8];
		values[0] = num;
		values[1] = mean;
		values[2] = median;
		values[3] = variance;
		values[4] = stddev;
		values[5] = avgabsdev;
		values[6] = min;
		values[7] = max;
		return values;
	}

}
