package org.cakelab.statistics;

import org.cakelab.utils.LongArray;


public class StatisticsLong implements Statistics {


	long num; /* number of samples */
	long min; 
	long max;
	double mean;
	long median;
	double stddev;
	double variance;
	double avgabsdev;

	public void update(LongArray values){
		num = values.size();
		max = Long.MIN_VALUE;
		min = Long.MAX_VALUE;
		
		double sum = 0;
		
		
		//
		// Min, max and mean
		//
		
		for (int i = 0 ; i < num; i++) {
			long val = values.a[i];
			if (val < min) min = val;
			if (val > max) max = val;
			sum += val;
		}
		
		mean = sum/num;
		
		
		//
		// variance, standard deviation and average absolute deviation
		//
		double absdevsum = 0;
		double squaresum = 0;
		double dev;
		for (int i = 0 ; i < num; i++) {
			long val = values.a[i];
			dev = val-mean;
			absdevsum += Math.abs(dev);
			squaresum += dev*dev;
		}
		variance = squaresum/num;
		stddev = Math.sqrt(variance);
		avgabsdev = absdevsum/num;

		//
		// median
		//
		values.sort();
		median = values.a[(int) (num/2)];
		
	}
	
	
	
	public static double mean(LongArray a) {
		double sum = 0;
		for (int i = 0; i < a.size(); i++) {
			sum += a.a[i];
		}
		return sum / a.size();
	}
	
	


	public long getNum() {
		return num;
	}



	public long getMinLong() {
		return min;
	}

	public double getMinDouble() {
		return (double)min;
	}



	public long getMaxLong() {
		return max;
	}

	public double getMaxDouble() {
		return (double)max;
	}



	public double getMean() {
		return mean;
	}


	public long getMedianLong() {
		return median;
	}

	
	public double getMedianDouble() {
		return (double)median;
	}


	public double getStddev() {
		return stddev;
	}


	public double getVariance() {
		return variance;
	}


	public double getAvgabsdev() {
		return avgabsdev;
	}
	
	public String toString() {
		StringBuffer result = new StringBuffer("num: " + num + ", ");
		result.append("min: " + min + ", ");
		result.append("max: " + max + ", ");
		result.append("mean: " + mean + ", ");
		result.append("median: " + median + ", ");
		result.append("stddev: " + stddev + ", ");
		result.append("variance: " + variance + ", ");
		result.append("avgabsdev: " + avgabsdev);
		return result.toString();
	}



	public void clear() {
		num = 0;
		max = Long.MIN_VALUE;
		min = Long.MAX_VALUE;
		mean = 0;
		median = 0;
		variance = 0;
		stddev = 0;
		avgabsdev = 0;
	}
	
	@Override
	public Number[] getValues() {
		Number[] values = new Number[8];
		values[0] = num;
		values[1] = mean;
		values[2] = median;
		values[3] = variance;
		values[4] = stddev;
		values[5] = avgabsdev;
		values[6] = min;
		values[7] = max;
		return values;
	}
}
