package org.cakelab.utils;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

public class Arguments {
	/**
	 * Merge to arrays with arguments.
	 * args2 override args1!
	 * 
	 * Note: It implicitly sorts the arguments alphabetically.
	 * 
	 * 
	 * @param args1
	 * @param args2
	 * @return
	 */
	public static String[] merge(String[] args1, String[] args2) {
		HashMap<String,String> map = new HashMap<String,String>(args1.length+ args2.length);
		int size = mapAddAllArgs(map, args1);
		size += mapAddAllArgs(map, args2);
		
		String[] keys = new String[1];
		keys = map.keySet().toArray(keys);
		Arrays.sort(keys, String.CASE_INSENSITIVE_ORDER);


		String[] result = new String[size];
		
		int i = 0;
		for (String key : keys) {

			result[i] = key;
			i++;
			
			String val = map.get(key);
			if (val != null) {
				result[i] = val;
				i++;
			}
		}
		
		return result;
	}

	
	public static String[] sort(String[] args) {
		return merge(args, new String[]{});
	}
	
	
	public static void set(String[] args, String key, int value) {
		set(args, key, Integer.toString(value));
	}

	public static void set(String[] args, String key, String value) {
		for (int i = 0 ; i < args.length; i++) {
			if (args[i].equals(key)) {
				i++;
				if (i >= args.length || args[i].startsWith("-")) throw new Error("Argument " + key + " has no value");

				args[i] = value;
			}
		}
	}


	private static int mapAddAllArgs(Map<String, String> map, String[] args) {
		int i = 0;
		int size = 0;
		while (i < args.length) {
			String arg = args[i];
			if (!arg.startsWith("-")) throw new Error("arguments have to start with '-'");
			
			i++;
			String val = null;
			if (i < args.length && !args[i].startsWith("-")) {
				val = args[i];
				i++;
			}
			if (!map.containsKey(arg)) {
				size++;
				if (val != null) size++;
			}
			map.put(arg,  val);
		}
		return size;
	}

}
