package org.cakelab.utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import org.cakelab.eval.mlatency.MLatencyMeasurementRun;
import org.cakelab.eval.model.LaunchConfig;

public class FileSystem {
	static final boolean OPEN_APPEND = true;

	public static final String propertiesSeparator = "_";

	public static void dump2File(InputStream in, String filename) throws IOException {
		FileOutputStream out = new FileOutputStream(filename);
		dump2stream(in, out);
		out.close();
	}

	public static String createFilename(String string) {
		String result = string.replaceAll(" ", propertiesSeparator);
		return result;
	}

	public static String basename(String executable) {
		String[] path = executable.split("/");
		
		for (int i = path.length-1; i>= 0; i--) {
			String base = path[i];
			if (path.equals("")) continue;
			else if (path.equals(".")) throw new Error(". is not an executable");
			else return base;
		}
		throw new Error("unable to find the basename of '" + executable + "'");
	}

	public static void dump2stream(InputStream in, OutputStream out) throws IOException {
		byte[] buf = new byte[1024];
		int read;
		while (0 < (read = in.read(buf))) {
			out.write(buf, 0, read);
		}
	}

	public static void createDirectory(String dir) {
		File d = new File(dir);
		if (!d.exists()) d.mkdirs();
	}

	
	public static String createDataLogDirectory(String targetDirectory) {
		String dir = targetDirectory + File.separator + "data";
		createDirectory(dir);
		return dir;
	}
	
	public static String createResultsLogDirectory(String targetDirectory) {
		String dir = targetDirectory + File.separator + "results";
		createDirectory(dir);
		return dir;
	}

	public static String createLaunchConfigDirectory(String basedir, LaunchConfig launch) {
		String dir = basedir + File.separator + launch.getArgumentsString(FileSystem.propertiesSeparator);
		createDirectory(dir);
		return dir;
	}

	public static String createRunDir(String datalogDir,
			MLatencyMeasurementRun run) {
		String dir = datalogDir + File.separator + run.getTimestamp();
		createDirectory(dir);
		return dir;
	}

	public static long runDirTimeStamp(File file) {
		String tmStr = file.getName();
		// TODO: date should be stored in a file
		Long date = Long.decode(tmStr);
		return date;
	}

	public static String getDirectory(String fn) {
		File f = new File (fn);
		return f.getParent();
	}

	public static String createPlotsDirectory(String dir) {
		dir = dir + File.separator + "/plots";
		createDirectory(dir);
		return dir;
	}

	public static String readText(String fn) throws IOException {
		StringBuffer text = new StringBuffer();
		FileInputStream fin = new FileInputStream(fn);
		int size = 1024;
		byte[] buffer = new byte[size];
		while (0 < (size = fin.read(buffer))) {
			// TODO: consider different character encodings in config file
			text.append(new String(buffer, 0, size));
		}
		fin.close();
		return text.toString();
	}

	public static boolean exists(String filename) {
		return new File(filename).exists();
	}

	

}
